c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%
       subroutine LECTUR
c      %%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       use inoutput
       USE H2_DATA
       USE HD_DATA
       USE SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c---------------------------------------------------------------------
c       lecture des especes chimiques considerees, de leurs abondances
c       initiales et des equations chimiques (appele depuis init).
c              avec ions negatifs eventuellement.
c---------------------------------------------------------------------

       implicit double precision (a-h,o-z)

       character (LEN=8) ref
       character (LEN=8) r1, r2, r3, p1, p2, p3, p4

        real (KIND=DP)       msp(natom+6)

c              set up molecular weight of elements in amu

c If 10 : Li and 19 : k
c       data msp / 1.0d0, 12.0d0, 14.0d0, 16.0d0, 4.0d0, 2.0d0,
c    1             13.0d0, 15.0d0, 18.0d0, 6.9d0, 19.0d0, 23.0d0,
c    2             24.3d0, 27.0d0, 28.1d0, 31.0d0, 32.0d0, 35.5d0,
c    3             39.1d0, 55.8d0, 0.0d0, 0.0d0, 0.0d0 , 0.0d0,
c    4             0.0d0, 0.0d0, 0.0d0 /
c If 10 : PAH and 19 : ca
        data msp / 1.0d0, 12.0d0, 14.0d0, 16.0d0, 4.0d0, 2.0d0,
     1             13.0d0, 15.0d0, 18.0d0, 666.0d0, 19.0d0, 23.0d0,
     2             24.3d0, 27.0d0, 28.1d0, 31.0d0, 32.0d0, 35.5d0,
     3             40.08d0, 55.8d0, 0.0d0, 0.0d0, 0.0d0 , 0.0d0,
     4             0.0d0, 0.0d0, 0.0d0 /

c*********************************************************************

c              correspondances entre indices et elements:

c        1  :  h
c        2  :  c
c        3  :  n
c        4  :  o
c        5  :  he
c        6  :  d
c        7  :  c13 (=c*)
c        8  :  n15 (=n*)
c        9  :  o18 (=o*)
c       10  :  li or PAH
c       11  :  f
c       12  :  na
c       13  :  mg
c       14  :  al
c       15  :  si
c       16  :  p
c       17  :  s
c       18  :  cl
c       19  :  k ou ca
c       20  :  fe
c       21  :  charge

c      print *, iread1, iread2, iwrit1, iwrit2, iscrn

       mol(0)  = zero
       enth(0) = zero

c   read species file and set up conservation array

       i = 1
       nn = 0
       nng = 0
       ni = 0
       nim = 0
       speci(0) = "        "

       fichier = TRIM(data_dir)//chimie
       open (iread1, file = fichier, status = 'old')

          read (iread1,1101)

   40     continue
          read (iread1,1010) l, speci(i), (ael(i,j), j=1,21),
     1            abin(i), enth(i)

c  identification des indices de toutes les especes jouant un role particulier

        if (speci(i) .eq. 'h'   .or. speci(i) .eq. 'H'   )   i_h = i
        if (speci(i) .eq. 'd'   .or. speci(i) .eq. 'D'   )   i_d = i
        if (speci(i) .eq. 'h:'  .or. speci(i) .eq. 'H:'  ) i_hgr = i
        if (speci(i) .eq. 'd:'  .or. speci(i) .eq. 'D:'  ) i_dgr = i
        if (speci(i) .eq. 'h2'  .or. speci(i) .eq. 'H2'  )  i_h2 = i
        if (speci(i) .eq. 'hd'  .or. speci(i) .eq. 'HD'  )  i_hd = i
        if (speci(i) .eq. 'h2:' .or. speci(i) .eq. 'H2:' )i_h2gr = i
        if (speci(i) .eq. 'he'  .or. speci(i) .eq. 'HE'  )  i_he = i
        if (speci(i) .eq. 'c'   .or. speci(i) .eq. 'C'   )   i_c = i
        if (speci(i) .eq. 'n'   .or. speci(i) .eq. 'N'   )   i_n = i
        if (speci(i) .eq. 'o'   .or. speci(i) .eq. 'O'   )   i_o = i
        if (speci(i) .eq. 's'   .or. speci(i) .eq. 'S'   )   i_s = i
        if (speci(i) .eq. 'si'  .or. speci(i) .eq. 'Si'  )  i_si = i
        if (speci(i) .eq. 'c*'  .or. speci(i) .eq. 'C*'  ) i_c13 = i
        if (speci(i) .eq. 'o*'  .or. speci(i) .eq. 'O*'  ) i_o18 = i
        if (speci(i) .eq. 'h2o' .or. speci(i) .eq. 'H2O' ) i_h2o = i
        if (speci(i) .eq. 'oh'  .or. speci(i) .eq. 'OH'  )  i_oh = i
        if (speci(i) .eq. 'c2'  .or. speci(i) .eq. 'C2'  )  i_c2 = i
        if (speci(i) .eq. 'co'  .or. speci(i) .eq. 'CO'  ) i_co0 = i
        if (speci(i) .eq. 'c*o' .or. speci(i) .eq. 'C*O' ) i_co1 = i
        if (speci(i) .eq. 'co*' .or. speci(i) .eq. 'CO*' ) i_co2 = i
        if (speci(i) .eq. 'c*o*'.or. speci(i) .eq. 'C*O*') i_co3 = i
        if (speci(i) .eq. 'cs'  .or. speci(i) .eq. 'CS'  )  i_cs = i

c   cas des ions de charge +1:

          if (ael(i,natom) .ge.  1 ) goto 60

c   cas des ions de charge -1:

          if (ael(i,natom) .le. -1 ) goto 80

c   fin de la liste des especes:

          if (l .eq. 888) goto 100

c       calcul du poids moleculaire de chaque espece chimique in amu
c       et du nombre d especes dans chaque fluide :
c       (nn neutres, ni ions, nspec = nn + ni especes)

c              neutres

c  Le cas de H2 adsorbe est particulier:
c     H2: = Somme Si fi ds**2
c     avec: Si = surface des grains de taille i
c           fi = fraction occupee par H2 (entre 0 et 1)
c           ds = dsite
c     La somme va de 1 a npg (i.e., l'echantillonage est celui
c                                de l'integration gaussienne)

          if (i_h2gr == i) then
            ii = i
            do mm=1,npg
              speci(i) = speci(ii)
              ael(i,:) = ael(ii,:)
              abin(i) = abin(ii)
              enth(i) = enth(ii)
              mol(i) = deux
              nng = nng + 1
              i = i + 1
            end do
            abin(ii:ii+npg-1) = abin(ii:ii+npg-1) / wwmg
          else

            sum = zero
            do j=1,natom-1
               sum = sum + ael(i,j) * msp(j)
            end do
            mol(i) = sum

            if (ni .eq. 0) then
              nn = i
            else
              nng = nng + 1
            endif

            i = i + 1

          endif
          goto 40

c              ions positifs

   60     continue
          sum = zero
          do j=1,natom-1
            sum = sum + ael(i,j) * msp(j)
          end do

          if (speci(i) .eq. 'h+'  .or. speci(i) .eq. 'H+'  )    i_hp = i
          if (speci(i) .eq. 'h3+'  .or. speci(i) .eq. 'H3+')   i_h3p = i
          if (speci(i) .eq. 'h2d+' .or. speci(i) .eq. 'H2D+') i_h2dp = i
          if (speci(i) .eq. 'c+'   .or. speci(i) .eq. 'C+' )    i_cp = i
          if (speci(i) .eq. 'c*+'  .or. speci(i) .eq. 'C*+' ) i_c13p = i
          if (speci(i) .eq. 'n+'   .or. speci(i) .eq. 'N+')     i_np = i
          if (speci(i) .eq. 'o+'   .or. speci(i) .eq. 'O+')     i_op = i
          if (speci(i) .eq. 's+'   .or. speci(i) .eq. 'S+')     i_sp = i
          if (speci(i) .eq. 'si+'  .or. speci(i) .eq. 'SI+')   i_sip = i
          if (speci(i) .eq. 'fe+'  .or. speci(i) .eq. 'FE+')   i_fep = i
          if (speci(i) .eq. 'hco+' .or. speci(i) .eq. 'HCO+') i_hcop = i
          mol(i) = sum
          ni = i - nn
          i = i + 1
          goto 40

c           ions negatifs

   80     continue
          sum = zero
          do j=1,natom-1
            sum = sum + ael(i,j) * msp(j)
          end do
          mol(i) = sum
          nim = i - nn - ni
          i = i + 1
          goto 40

  100     nspec = i - 1

c              ions positifs

          sumn = zero
          do i=nn+1,nn+ni
            sumn = sumn + abin(i)
          end do
          xni0 = sumn

c              ions negatifs

          sumn = zero
          do i=nn+ni+1,nspec-nng
            sumn = sumn + abin(i)
          end do
          xnim0 = sumn

c     addition des electrons (sp(21)), photons (sp(22)), crp (sp(23)), phosec (sp(24))
c         grains (sp(25)), 2h (sp(26))          aux especes considerees:
          do i=1,6
            ii = nspec + i
            jj = i + natom - 1
            speci(ii) = sp(jj)
            mol(ii) = msp(jj)
          end do
          enth(nspec+1) = zero
          enth(nspec+2) = 13.0d0 / calev
          enth(nspec+3) = 20.0d0 / calev
          enth(nspec+4) = 13.0d0 / calev
          enth(nspec+5) = zero
          enth(nspec+6) = zero

c n2 : nbre (d especes chimiques + electron) + photon + crp + phosec + grains + 2h
c       les electrons sont comptes comme une espece et donc ajoutes
          nspec = nspec + 1

c              abondance initiale des electrons.
          if (xni0 .lt. xnim0) write (iscrn,9001) xni0, xnim0
          abin(nspec) = xni0 - xnim0
          lcon(nspec) = natom

c       calcul des abondances totales de chaque atome:

          abtot = zero
          do i=1,nspec-1
            do l=1,natom-1
              abtot(l) = abtot(l) + ael(i,l) * abin(i)
            end do
          end do

          write (iwrit2,2000)
          do l=1,nspec
            write (iwrit2,2010) l, speci(l), mol(l), abin(l),
     1             enth(l)
          end do

c  tm1h(i) est l inverse du temps que mets l espece i a faire un "HOP" (!)
      tm1h = 0.0d0
      if (i_hgr .ne. 0) then
        tm1h(i_hgr) = 5.0d10
      endif
      if (i_dgr .ne. 0) then
        tm1h(i_dgr) = 5.0d10 / sqrt(deux)
      endif

          write (iwrit2,2020) nn, ni, nim, nng, nspec

c************************************************************************
c       lecture des reactions chimiques
c          avec mise en ordre
c************************************************************************

          write (iwrit2,2050)
          i = 1
  200     do j=0,7
            react(i,j) = 999
          end do

          read (iread1,1020) ref, r1, r2, p1, p2, p3, p4,
     1              gamm(i), alpha(i), bet(i), itype(i)


          if (ref .eq. '99999') goto 300

          if (r1 .eq. '        ') then
            goto 200
          endif

c pour chimie pepe et eviter les duplications (provided by Evelyne)
           if (ref .eq. 'deja ')  then
             go to 200
           endif
c          if (bet(i) .lt. 0.d0)  then
c            bet(i) = 0.d0
c          endif
c fin modif

c  ATTENTION : speci commence a 0 !!!

          do j=0,nspec+5
            if (r1 == speci(j)) then
              react(i,1) = j
              exit
            endif
          end do
          do j=0,nspec+4
            if (r2 == speci(j)) then
              react(i,2) = j
              react(i,3) = 0
              exit
            endif
          end do
          if (r2 == speci(nspec+5)) then
            react(i,2) = i_h
            react(i,3) = i_h
          endif
          do j=0,nspec+5
            if (p1 == speci(j)) then
              react(i,4) = j
              exit
            endif
          end do
          do j=0,nspec+5
            if (p2 == speci(j)) then
              react(i,5) = j
              exit
            endif
          end do
          do j=0,nspec+5
            if (p3 == speci(j)) then
              react(i,6) = j
              exit
            endif
          end do
          do j=0,nspec+5
            if (p4 == speci(j)) then
              react(i,7) = j
              exit
            endif
          end do

          if (r2 .eq. sp(natom+6))  react(i,2) = 0
          if (p2 .eq. sp(natom+6))  react(i,5) = 0
          if (p3 .eq. sp(natom+6))  react(i,6) = 0
          if (p4 .eq. sp(natom+6))  react(i,7) = 0

c       recherche des reactions incompletes

          do j=1,7
            if (react(i,j) .eq. 999)  then
              write (iwrit2,2060) i, (react(i,jj), jj=1,7)
              write (iscrn,2060) i, (react(i,jj), jj=1,7)
              goto 200
            endif
          end do

          if(react(i,5) .eq. 0) then
            react(i,0) = 3
          else if(react(i,6) .eq. 0) then
            react(i,0) = 4
          else if(react(i,7) .eq. 0) then
            react(i,0) = 5
          else
            react(i,0) = 6
          endif

c----calcul de la somme des masses des produits de chaque reaction
c    et du bilan net d energie de chaque reaction (en ev)
c    convention: positif ==> exothermique
c                negatif ==> endothermique
c                ******* ==> donnees insuffisantes

      if (react(i,5) .ne. nspec+1) then
        dreact = enth(react(i,1)) + enth(react(i,2))
     1         + enth(react(i,3))
        dprod = zero
        kj = react(i,0)
        do j=4,kj+1
          iprod = react(i,j)
          if (iprod .ne. nspec+1) then
            dprod = dprod + enth(iprod)
          endif
        end do
        de(i) = dreact - dprod
        de(i) = de(i) * calev
      else
        de(i) = zero
      endif

c    ********************************
c----ecriture des reactions chimiques
c    ********************************

          i = i + 1

c  On ajoute ICI la "pseudo-reaction" limitant l'adsorption de H2 sur les grains
c  et on multiplie les reactions impliquant la distribution de taille des grains

      if (itype(i-1) == 113) then
        gamm(i) = gamm(i-1)
        alpha(i) = alpha(i-1)
        bet(i) = bet(i-1)
        itype(i) = itype(i-1) + 10
        react(i,0) = react(i-1,0)
        react(i,1) = react(i-1,1)
        react(i,2) = i_h2gr
        react(i,3) = 0
        react(i,4) = i_h2
        react(i,5) = react(i-1,5)
        react(i,6) = react(i-1,6)
        react(i,7) = react(i-1,7)
        i = i + 1
        do mm=1,npg-1
          gamm(i) = gamm(i-2)
          alpha(i) = alpha(i-2)
          bet(i) = bet(i-2)
          itype(i) = itype(i-2)
          react(i,0) = react(i-2,0)
          react(i,1) = react(i-2,1)
          react(i,2) = react(i-2,2)
          react(i,3) = 0
          react(i,4) = react(i-2,4) + 1
          react(i,5) = react(i-2,5)
          react(i,6) = react(i-2,6)
          react(i,7) = react(i-2,7)
          i = i + 1

          gamm(i) = gamm(i-2)
          alpha(i) = alpha(i-2)
          bet(i) = bet(i-2)
          itype(i) = itype(i-2)
          react(i,0) = react(i-2,0)
          react(i,1) = react(i-2,1)
          react(i,2) = react(i-2,2) + 1
          react(i,3) = 0
          react(i,4) = react(i-2,4)
          react(i,5) = react(i-2,5)
          react(i,6) = react(i-2,6)
          react(i,7) = react(i-2,7)
          i = i + 1
        end do
      else if (itype(i-1) == 117) then
        do mm=1,npg-1
          gamm(i) = gamm(i-1)
          alpha(i) = alpha(i-1)
          bet(i) = bet(i-1)
          itype(i) = itype(i-1)
          react(i,0) = react(i-1,0)
          react(i,1) = react(i-1,1) + 1
          react(i,2) = react(i-1,2)
          react(i,3) = 0
          react(i,4) = react(i-1,4)
          react(i,5) = react(i-1,5)
          react(i,6) = react(i-1,6)
          react(i,7) = react(i-1,7)
          i = i + 1
        end do
      else if (itype(i-1) == 118) then
        do mm=1,npg-1
          gamm(i) = gamm(i-1)
          alpha(i) = alpha(i-1)
          bet(i) = bet(i-1)
          itype(i) = itype(i-1)
          react(i,0) = react(i-1,0)
          react(i,1) = react(i-1,1) + 1
          react(i,2) = react(i-1,2)
          react(i,3) = 0
          react(i,4) = react(i-1,4)
          react(i,5) = react(i-1,5)
          react(i,6) = react(i-1,6)
          react(i,7) = react(i-1,7)
          i = i + 1
        end do
      end if

          goto 200

  300     continue
          neqt = i - 1

c     debut ici de la construction de la chimie isotopee

      nreact = neqt

c *************************************************************************
c                 construction de la chimie isotopee
c *************************************************************************

c   convention :    dans la liste des especes, chaque espece isotopee doit
c   ----------      suivre immediatement l espece mere
c                   dans le cas ou l espece contient de l oxygene et du
c                   carbone (c ou c*) ==> l espece isotopee: o* suit c*
c                   ex :    hco2
c                           hc*o2
c                           hco*o
c                           hc*o*o

c *************************************************************************

c--- y-a-t il des isotopes dans la liste des especes neutres (c* puis o*) ?

c-------------------------------------------------------------------------
c --- variable test : i_c13 > 0  ==>  il y a du c13 dans la chimie
c                               ==>  il faut traiter l excitation de c*o

c                     i_c13 = 0  ==>  il n'y a pas de c13

c                     i_o18 > 0  ==>  il y a du o18 dans la chimie
c                               ==>  il faut traiter l excitation de co*, c*o*

c                     i_o18 = 0  ==>  il n'y a pas de o18
c-------------------------------------------------------------------------

c--- y-a-t il des isotopes dans la liste des especes neutres (c* et o*) ?
c--- non ===> la liste des reactions est close (=go to 400)

      if ((i_c13.eq.0).and.(i_o18.eq.0))  goto 400

c---------
c --- c*
c---------

      if (i_c13.gt.0)  goto 320

c--- non ===> on examine la presence eventuelle de o* (=go to 401)
      goto 401

c--- oui ===> alors on appelle la subroutine ISOTC

  320 call ISOTC (nreact, idiff)
  401 continue

c---------
c --- o*
c---------
      if (i_o18.gt.0)  goto 330

c--- non ===> la liste des reactions est close (=go to 402)

      goto 402

c--- oui ===> alors on appelle la subroutine ISOTO

  330 call ISOTO (nreact,idiff)
  402 continue

c----------------------------------------------------------------------------

c         neqt   = nombre total de reactions sans isotope (pour l instant)
c         nreact = numero de la reaction isotopee
c         idiff  = numero de la derniere reaction differentielle

c----------------------------------------------------------------------------
c  fin de l examen des reactions et de la construction de la chimie isotopee
c----------------------------------------------------------------------------

      write (iwrit2,2070)  i_c13, i_o18

      do 340 ii=neqt+1,nreact

c-----------------------------------------------------------------------------
c---calcul de la somme des masses des produits de chaque reaction
c   et du bilan net d energie de chaque reaction isotopee (en ev)
c-----------------------------------------------------------------------------

      dreact = zero
      dprod = zero
      do jj=1,3
        ireact = react(ii,jj)
        dreact = dreact + enth(ireact)
      end do
      kjj = react(ii,0) + 1
      do jj=4,kjj
        iprod = react(ii,jj)
        if (iprod .ne. nspec+1) then
          dprod = dprod + enth(iprod)
        endif
      end do
      de(ii) = dreact - dprod
      de(ii) = de(ii) * calev

c-----------------------------------------------------------------------------
c---ecriture des reactions isotopees
c-----------------------------------------------------------------------------

       r1 = speci (react(ii,1))
       r2 = speci (react(ii,2))
       r3 = speci (react(ii,3))
       p1 = speci (react(ii,4))
       p2 = speci (react(ii,5))
       p3 = speci (react(ii,6))
       p4 = speci (react(ii,7))

      ref = 'isotp   '
c-----------------------------------------------------------------------------

  340 continue

c--- nombre total de reactions : neqt (normales + isotopees)

  400 continue

      neqt = nreact

      write (iwrit2,2030) neqt

c   Ajout des reactions de photodissociation et photoionisation traitee explicitement
c   Elles correspondent a un type independant: itype = 7

c    c:
          neqt = neqt + 1
          react(neqt,0) = 4
          react(neqt,1) = i_c
          react(neqt,2) = nspec + 1
          react(neqt,3) = 0
          react(neqt,4) = i_cp
          react(neqt,5) = nspec
          react(neqt,6) = 0
          react(neqt,7) = 0
          itype(neqt) = 7
          dreact = enth(react(neqt,1)) + enth(react(neqt,2))
          dprod = enth(react(neqt,4)) + enth(react(neqt,5))
          de(neqt) = dreact - dprod
          de(neqt) = de(neqt) * calev
          ref = 'phot    '
          r1 = 'c       '
          r2 = 'photon  '
          r3 = '        '
          p1 = 'c+      '
          p2 = 'electr  '
          p3 = '        '
          p4 = '        '

c    c*:
          if (i_c13 .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_c13
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_c13p
            react(neqt,5) = nspec
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            dreact = enth(react(neqt,1)) + enth(react(neqt,2))
            dprod = enth(react(neqt,4)) + enth(react(neqt,5))
            de(neqt) = dreact - dprod
            de(neqt) = de(neqt) * calev
            ref = 'phot    '
            r1 = 'c*      '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'c*+     '
            p2 = 'electr  '
            p3 = '        '
            p4 = '        '
          endif

c    s:
          if (i_s .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_s
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_sp
            react(neqt,5) = nspec
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            dreact = enth(react(neqt,1)) + enth(react(neqt,2))
            dprod = enth(react(neqt,4)) + enth(react(neqt,5))
            de(neqt) = dreact - dprod
            de(neqt) = de(neqt) * calev
            ref = 'phot    '
            r1 = 's       '
            r2 = 'photon  '
            r3 = '        '
            p1 = 's+      '
            p2 = 'electr  '
            p3 = '        '
            p4 = '        '
          endif

c Pour la photodissociation de H2, HD et CO, on compte 1 eV par reaction.

c    h2:
          if (itrh2 .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_h2
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_h
            react(neqt,5) = i_h
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            de(neqt) = un
            ref = 'phot    '
            r1 = 'h2      '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'h       '
            p2 = 'h       '
            p3 = '        '
            p4 = '        '
          endif

c    hd:
          if (itrhd .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_hd
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_h
            react(neqt,5) = i_d
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            ref = 'phot    '
            de(neqt) = un
            r1 = 'hd      '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'h       '
            p2 = 'd       '
            p3 = '        '
            p4 = '        '
          endif

c    co:
          if (itrco0 .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_co0
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_c
            react(neqt,5) = i_o
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            de(neqt) = un
            ref = 'phot    '
            r1 = 'co      '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'c       '
            p2 = 'o       '
            p3 = '        '
            p4 = '        '
          endif

c    c*o:
          if (itrco1 .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_co1
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_c13
            react(neqt,5) = i_o
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            de(neqt) = un
            ref = 'phot    '
            r1 = 'c*o     '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'c*      '
            p2 = 'o       '
            p3 = '        '
            p4 = '        '
          endif

c    co*:
          if (itrco2 .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_co2
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_c
            react(neqt,5) = i_o18
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            de(neqt) = un
            ref = 'phot    '
            r1 = 'co*     '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'c       '
            p2 = 'o*      '
            p3 = '        '
            p4 = '        '
          endif

c    c*o*:
          if (i_co3 .ne. 0) then
            neqt = neqt + 1
            react(neqt,0) = 4
            react(neqt,1) = i_co3
            react(neqt,2) = nspec + 1
            react(neqt,3) = 0
            react(neqt,4) = i_c13
            react(neqt,5) = i_o18
            react(neqt,6) = 0
            react(neqt,7) = 0
            itype(neqt) = 7
            de(neqt) = un
            ref = 'phot    '
            r1 = 'c*o*    '
            r2 = 'photon  '
            r3 = '        '
            p1 = 'c*      '
            p2 = 'o*      '
            p3 = '        '
            p4 = '        '
          endif

c--- reordonnement des reactants par numero d especes croissant
c         (pour n'avoir en memoire qu'une matrice triangulaire)

      do i=1,neqt
        if (react(i,1) .gt. react(i,2) .and.
     1      react(i,2) .ne. 0)  then
          l = react(i,1)
          react(i,1) = react(i,2)
          react(i,2) = l
        endif
      end do

      close(iread1)

c    c est ici qu il faut reordonner les reactions.

      call ORDON

c    initialisation du tableau ikmat
c    liste des reactions utiles pour chaque espece

      do i=0,nspec+5
        ikmat(i,0) = 0
      end do

c  Attention: pour la pseudo-reaction 123, il ne faut compter H2
c      qu'une fois : du cote de la formation

c JLB - 15 march 2004 - revoir le role de ikmat...

      do i=1,neqt
        ik = react(i,0)
        if (itype(i) == 123) then
          j0 = 2
        else
          j0 = 1
        endif
        do j=j0,ik+1
          iis = react(i,j)
          if (iis == 0) then
            cycle
          endif
          n = ikmat(iis,0) + 1
          ikmat(iis,n) = i
          ikmat(iis,0) = n
        end do
      end do

       write (iwrit2,2031) neqt

c*********************************************************************

c  probabilite de former h2, hd dans l etat nv, nj sur
c  les grains. Le rapport O/P suit les poids statistiques
c  i.e.: 3/4 ortho et 1/4 para

c  6 XI 2003 : Let's try a different O/P on grain..

       ospg = 3.0_dp
c      ospg = 1.0_dp

       print *, '----------- Formation de H2 et HD ------------'

       if (iforh2 .eq. 0) then

c  La temperature d'excitation est ajustee de telle sorte que l'energie
c     moyenne soit exactement de 1.49 eV, quel que soit le nombre de niveau de H2

         eform = everg * de(ifh2gr) / (trois * xk)

         tform = eform
         tf_min = 0.0_dp
         tf_max = deux * eform
         res = 1.0_dp

         do while ((abs(res) > 1.0e-3_DP) .AND.
     1             (abs(tform-tf_max) > 1.0e-3))
           res = 0.0_DP
           do lev=1,l0h2
             ij = njlev(lev)
             if (mod(ij,2) == 0) then
               res = res + (deux*ij+un) * (eform - elhm(lev))
     1                   * exp(-elhm(lev)/tform)
             else
               res = res + (deux*ij+un) * ospg * (eform - elhm(lev))
     1                   * exp(-elhm(lev)/tform)
             endif
           end do
           if (res < 0.0_dp) then
             tf_max = tform
             tform = 0.5_DP * (tf_min + tf_max)
           else
             tf_min = tform
             tform = 0.5_DP * (tf_min + tf_max)
           endif
         end do

         eform = tform

         write (iscrn,2101) de(ifh2gr), eform
         write (iwrit2,2101) de(ifh2gr), eform

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           if (mod(ij,2) == 0) then
             pophm(iv,ij) = (deux*ij+un)
     1                    * exp(-elhm(lev)/eform)
           else
             pophm(iv,ij) = (deux*ij+un) * ospg
     1                    * exp(-elhm(lev)/eform)
           endif
           ech = ech + pophm(iv,ij)
         end do

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           pophm(iv,ij) = pophm(iv,ij) / ech
         end do

         eformd = eform

         if (i_hd .ne. 0) then
           ech = 0
           do ij = 0,mjhd
            do iv=0,nvxhd
              lev = levhd(iv,ij)
              if (lev .eq. 0) cycle
              pophd(iv,ij) = (deux*ij+un) * exp(-elhd(lev)/eformd)
              ech = ech + pophd(iv,ij)
            end do
           end do

           dummy = zero
           do ij = 0,mjhd
            do iv = 0,nvxhd
              lev = levhd(iv,ij)
              if (lev .eq. 0) cycle
              pophd(iv,ij) = pophd(iv,ij) / ech
            end do
           end do
         endif

       else if (iforh2 .eq. 1) then

c  autre hypothese pour la formation de h2:
c  equipartition et distribution thermique de de(ifh2gr)/3

         eform = everg * de(ifh2gr) / (trois * xk)
         if (i_hd .ne. 0) then
           eformd = everg * de(ifhdgr) / (trois * xk)
         endif

c black & van dishoeck, formation phi = 1
c     apj, 322, 412 (1987), eform = 12098 cm-1

         write (iscrn,2101) de(ifh2gr), eform
         write (iwrit2,2101) de(ifh2gr), eform

         ech = zero

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           if (mod(ij,2) == 0) then
             pophm(iv,ij) = (deux*ij+un)
     1                    * exp(-elhm(lev)/eform)
           else
             pophm(iv,ij) = (deux*ij+un) * trois
     1                    * exp(-elhm(lev)/eform)
           endif
           ech = ech + pophm(iv,ij)
         end do

         do lev=1,l0h2
           ij = njlev(lev)
           iv = nvlev(lev)
           pophm(iv,ij) = pophm(iv,ij) / ech
         end do

c         print *, echp, echo, echp+echo

         if (i_hd .ne. 0) then
           ech = 0
           do 14 ij = 0,mjhd
            do 11 iv=0,nvxhd
              lev = levhd(iv,ij)
              if (lev .eq. 0) goto 11
              pophd(iv,ij) = (deux*ij+un) * exp(-elhd(lev)/eformd)
              ech = ech + pophd(iv,ij)
   11       continue
   14      continue

           dummy = zero
           do 12 ij = 0,mjhd
            do 13 iv = 0,nvxhd
              lev = levhd(iv,ij)
              if (lev .eq. 0) goto 13
              pophd(iv,ij) = pophd(iv,ij) / ech
   13       continue
   12      continue
         endif

       else if (iforh2 .eq. 2) then

         pophm(14,0) = 0.25d0
         pophm(14,1) = 0.75d0

         if (i_hd .ne. 0) then
           pophd(14,0) = 0.25d0
           pophd(14,1) = 0.75d0
         endif

       else if (iforh2 .eq. 3) then

c  tf (temperature de formation = 65 k
c   d apres bvd, proche de tgrain (?)

         pophm(6,0) = 0.245284557d+0
         pophm(6,1) = 0.749823809d+0
         pophm(6,2) = 0.471552089d-2
         pophm(6,3) = 0.176384812d-3
         pophm(6,4) = 0.247192595d-7
         pophm(6,5) = 0.265905215d-10
         pophm(6,6) = 0.132664312d-15
         pophm(6,7) = 0.629879296d-20

         if (i_hd .ne. 0) then
           pophd(6,0) = 0.245284557d+0
           pophd(6,1) = 0.749823809d+0
           pophd(6,2) = 0.471552089d-2
           pophd(6,3) = 0.176384812d-3
           pophd(6,4) = 0.247192595d-7
           pophd(6,5) = 0.265905215d-10
           pophd(6,6) = 0.132664312d-15
           pophd(6,7) = 0.629879296d-20
         endif

       else

         write (iscrn,8888)
 8888    format (/,5x,' mauvaise valeur de iforh2')

       endif

       dumy = zero
       dummy = zero
       do lev=1,l0h2
         ij = njlev(lev)
         iv = nvlev(lev)
c        print *, iv, ij, pophm(iv,ij), elhm(lev)
         dumy = dumy + pophm(iv,ij) * elhm(lev)
         dummy = dummy + pophm(iv,ij)
       end do
       emoyh2 = dumy

       do lev=l0h2+1,nlevh2
         ij = njlev(lev)
         iv = nvlev(lev)
c        print *, iv, ij, pophm(iv,ij), elhm(lev)
         dummy = dummy + pophm(iv,ij)
       end do

       write (iscrn,*) 'sum(pophm):', dummy
       write (iscrn,2102) eform, emoyh2
       write (iwrit2,2102) eform, emoyh2

       dumy = zero
       dummy = zero
       do 9755 lev=1,l0hd
         ij = njhd(lev)
         if (ij .gt. mjhd) goto 9755
         iv = nvhd(lev)
c        print *, iv, ij, pophd(iv,ij), elhd(lev)
         dumy = dumy + pophd(iv,ij) * elhd(lev)
         dummy = dummy + pophd(iv,ij)
 9755  continue
       emoyhd = dumy

       levhd1 = l0hd+1
       do 9756 lev=levhd1,nlevhd
         ij = njhd(lev)
         if (ij .gt. mjhd) goto 9756
         iv = nvhd(lev)
c        print *, iv, ij, pophd(iv,ij), elhd(lev)
         dummy = dummy + pophd(iv,ij)
 9756  continue

       write (iscrn,*) 'sum(pophd):', dummy
       write (iscrn,2104) eformd, emoyhd
       write (iwrit2,2104) eformd, emoyhd

c---------------------------------------------------------------------
 1101  format (/)
 1010  format (i3,2x,a8,1x,2i2,3i1,1x,4i1,1x,5i1,1x,6i1,1x,i2
     1        ,1x,d10.3,f9.3,1x,a16)
 1020  format (a5,1x,5a8,a7,d8.2,1x,f5.2,1x,f8.1,i4)
 2000  format (1x,//,3x,'Included species :',/,
     1         23x,'weight   ab_init   enthalpy',/)
 2010  format (5x,i3,5x,a8,1x,f5.1,2x,1pe12.4,2x,0pf7.3,1x,a16)
 2020  format (//,3x,i3,' neutral species',
     1          /,3x,i3,' positively ionised species',
     2          /,3x,i3,' negatively ionised species',
     3          /,3x,i3,' species in grain mantles',
     4          /,3x,i3,' chemical species (with electrons)',/)
 2030  format (/,3x,i4,' chemical reactions (without photo-rxns)',/)
 2031  format (/,3x,i4,' chemical reactions (with photo-rxns)',//)
 2050  format (3x,'Chemical reactions',/,
     1         63x,'gamma',3x,'alpha',4x,'beta',3x,'deltae',/)
 2060  format (5x,'WARNING, error in reactions, line:',7i4)
 2070  format (//,10x,'Isotopic reactions',5x,'i_c13 =',i2,' i_o18 ='
     1         ,i2,/,10x,19('-'),/)
 2101  format (1x,'H2 formation enthalpie :',f10.4,' eV',
     1          /,1x,'including internal energy:'
     2           ,f12.2,' K')
 2102  format (1x,'eformh:',f12.2,' K  emoyh2:',f12.2,' K')
 2104  format (1x,'eformd:',f12.2,' K  emoyhd:',f12.2,' K')
 9001  format (5x,'attention, ions + :',1pd9.2,' ions - :',1pd9.2)
c---------------------------------------------------------------------

       return
       end

c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        subroutine ORDON

        use constantes
        use auxiliaries
        USE SP_DATA
        USE STR_DATA
        USE CHEM_DATA

        implicit double precision (a-h,o-z)

        character (LEN=8) r1, r2, r3, p1, p2, p3, p4

        dimension dee(nqm), lreact(nqm,0:7), ntype(nqm)
        dimension gammm(nqm), palph(nqm), bett(nqm)

c identify and re-order the chemical reactions:

c  WARNING : Type 6 reactions are treated BEFORE type 5
c            (this is because they result from a splitting of type 4)

c (0) formation of h2 and hd on grains [old style] (nh2for)
c (1) cosmic ray processes (ncrion)
c (2) secondary photon processes (npsion)
c (3) radiative association  (nasrad)
c (4) gas phase reactions (nrest)
c (6) Endothermal reactions with H2 (nh2endo) (NEW!)
c (5) photoreactions (external fit) (nphot_f)
c (7) photoreactions (direct integration) (nphot_i)
c (8) 3 body reactions
c (101...) exceptions (ncpart)

c (11) reactions on grain surface (ngsurf)
c (12) photoreactions on grain surface (ngphot)
c (13) adsorption on to grains (ngads)
c (14) neutralization (ngneut)
c (15) explosive desorption from grains (nggdes)
c (16) cosmic ray induced desorption from grains (ncrdes)
c (17) photodesorption from grains (nphdes)
c (18) evaporation from grains (ngevap)

c (111) to (118) same processes taking grain size distribution
c                into account

c -------------------------------------------------------
c the do loops are over all considered chemical reactions:
c -------------------------------------------------------

      nh2for = 0
      ncrion = 0
      npsion = 0
      nasrad = 0
      nrest = 0
      nh2endo = 0
      nphot_f = 0
      nphot_i = 0
      n3body = 0
      ncpart = 0
      ngsurf = 0
      ngphot = 0
      ngads = 0
      ngneut = 0
      nggdes = 0
      ncrdes = 0
      nphdes = 0
      ngevap = 0

      numero = 0

c (0) formation of h2 and hd on grains [old style] (nh2for)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 0) then
          nh2for = nh2for + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (1) cosmic ray processes (ncrion)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 1) then
          ncrion = ncrion + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c  --- le chauffage par les cosmiques est traite dans source.
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (2) secondary photon processes (npsion)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 2) then
          npsion = npsion + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          if (ael(react(i,1),10) == 1) then
            dee(numero) = zero
          else
            dee(numero) = de(i)
          endif
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (3) radiative association  (nasrad)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 3) then
          nasrad = nasrad + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) =  bet(i)
c  --- tout part dans le photon...
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (4) gas phase reactions (nrest)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 4) then
          nrest = nrest + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (6) Endothermal reactions with H2 (nh2endo)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 6) then
          nh2endo = nh2endo + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (5) photoreactions (nphot_f)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 5) then
          nphot_f = nphot_f + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c  --- guillaume dixit.
c         dee(numero) = zero
c  --- Evelyne dixit.
          dee(numero) = de(i) * 0.1d0
c  Cas particulier des PAH - Le chauffage est deja compte!
          if (ael(react(i,1),10) == 1) then
            dee(numero) = 0.0d0
          endif
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (7) photoreactions (direct integration) (nphot_i)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 7) then
          nphot_i = nphot_i + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c  --- guillaume dixit.
c         dee(numero) = zero
c  --- Evelyne dixit.
          dee(numero) = de(i) * 0.1d0
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do

          if (react(i,1) .eq. i_h2) then
            idh2ph = numero
          endif
          if (react(i,1) .eq. i_hd) then
            idhdph = numero
          endif
          if (react(i,1) .eq. i_co0) then
            idco0p = numero
          endif
          if (react(i,1) .eq. i_co1) then
            idco1p = numero
          endif
          if (react(i,1) .eq. i_co2) then
            idco2p = numero
          endif
          if (react(i,1) .eq. i_co3) then
            idco3p = numero
          endif
          if (react(i,1) .eq. i_c) then
            idcph = numero
          endif
        endif
      end do

c (8) 3 body reactions

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 8) then
          n3body = n3body + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
c Which enthapy for 3 body?
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (101...) exceptions (ncpart)

      do i=1,neqt
        ity = itype(i)
        if (ity > 100 .and.
     1      ity < 110) then
          ncpart = ncpart + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = de(i)
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (11) reactions on grain surface (ngsurf)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 11 .or. ity .eq. 111) then
          ngsurf = ngsurf + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif

        if (react(i,1) .eq. i_hgr .and.
     1      react(i,2) .eq. i_hgr) then
          dee(numero) = de(i)
          ifh2gr = numero
        endif

        if (react(i,1) .eq. i_hgr .and.
     1      react(i,2) .eq. i_dgr .or.
     2      react(i,1) .eq. i_dgr .and.
     3      react(i,2) .eq. i_hgr) then
          dee(numero) = de(i)
          ifhdgr = numero
        endif

      end do

c (12) photoreactions on grain surface (ngphot)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 12 .or. ity .eq. 112) then
          ngphot = ngphot + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (13) adsorption on to grains (ngads)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 13 .or.
     1      ity .eq. 113 .or. ity .eq. 123) then
          ngads = ngads + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do

c STicking coef de H:
          if (react(i,1) .eq. i_h) then
            iadhgr = numero
          endif

c STicking coef de D:
          if (react(i,1) .eq. i_d) then
            iaddgr = numero
          endif

        endif
      end do

c (14) neutralization (ngneut)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 14 .or. ity .eq. 114) then
          ngneut = ngneut + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (15) explosive desorption from grains (nggdes)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 15 .or. ity .eq. 115) then
          nggdes = nggdes + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (16) direct cosmic ray induced desorption from grains (ncrdes)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 16 .or. ity .eq. 116) then
          ncrdes = ncrdes + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (17) photodesorption from grains (UV + PHOSEC) (nphdes)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 17 .or. ity .eq. 117) then
          nphdes = nphdes + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

c (18) evaporation from grains (ngevap)

      do i=1,neqt
        ity = itype(i)
        if (ity .eq. 18 .or. ity .eq. 118) then
          ngevap = ngevap + 1
          numero = numero + 1
          gammm(numero) = gamm(i)
          palph(numero) = alpha(i)
          bett(numero) = bet(i)
          dee(numero) = zero
          ntype(numero) = itype(i)
          do j = 0,7
            lreact(numero,j) = react(i,j)
          end do
        endif
      end do

      do i = 1,neqt
        gamm(i) = gammm(i)
        alpha(i) = palph(i)
        bet(i) = bett(i)
        de(i) = dee(i)
        itype(i) = ntype(i)
        do j = 0,7
          react(i,j) = lreact(i,j)
        end do
          r1=sp(natom+6)
          r2=sp(natom+6)
          r3=sp(natom+6)
          p1=sp(natom+6)
          p2=sp(natom+6)
          p3=sp(natom+6)
          p4=sp(natom+6)
        do j=1,nspec+6
          if(react(i,1).eq.j) r1=speci(j)
          if(react(i,2).eq.j) r2=speci(j)
          if(react(i,3).eq.j) r3=speci(j)
          if(react(i,4).eq.j) p1=speci(j)
          if(react(i,5).eq.j) p2=speci(j)
          if(react(i,6).eq.j) p3=speci(j)
          if(react(i,7).eq.j) p4=speci(j)
        end do
        if (itype(i) /= 7) then
          write(iwrit2, 2040) r1,r2,r3,p1,p2,p3,p4,
     1                 gamm(i),alpha(i),bet(i),de(i),itype(i),i
        else
          write(iwrit2, 2041) r1,r2,r3,p1,p2,p3,p4,
     1                 "   Direct integration",de(i),itype(i),i
        endif
      end do

 2040  format (2x,'     ',1x,a8,'+ ',a8,'+ ',a8,'= ',3(a8,1x),a8,
     1         1pe8.2,2x,0pf6.2,2x,f8.1,2x,f9.3,3x,i4,i5)
 2041  format (2x,'     ',1x,a8,'+ ',a8,'+ ',a8,'= ',3(a8,1x),a8,
     1         a25,f9.3,3x,i4,i5)

        return
        end
