
! Copyright (C) 2002 - Observatoire de Paris/Meudon
!                      Universite Paris 7
!                      CNRS (France)
!                      Contact : Jacques.Lebourlot@obspm.fr
!
! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License version 2
! as published by the Free Software Foundation.
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

MODULE CONVER

  use constantes

! Convergence variables

  REAL (KIND=DP), PUBLIC                              :: varmax       ! convergence tolerance, target
  REAL (KIND=DP), PUBLIC                              :: thevar       ! convergence tolerance, done
  REAL (KIND=DP), PUBLIC                              :: dtaumin      ! min relative variation of tau
  REAL (KIND=DP), PUBLIC                              :: errx         ! relative error on abundances
  INTEGER                                             :: mauvai       ! identity of worse species
  REAL (KIND=DP), PUBLIC                              :: ceqh2        ! relative error on H2 levels
  REAL (KIND=DP), PUBLIC                              :: ceqhd        ! relative error on HD levels
  REAL (KIND=DP), PUBLIC                              :: bthrel       ! relative error on thermal balance

! Local variables (used for current iteration)

  REAL (KIND=DP), PUBLIC                              :: temp         ! Temperature
  REAL (KIND=DP), PUBLIC                              :: av           ! Av
  REAL (KIND=DP), PUBLIC                              :: abelec       ! Electron abundance

! Backup variables (for relative variations)

  REAL (KIND=DP), PUBLIC                              :: oldhat       ! H
  REAL (KIND=DP), PUBLIC                              :: oldhmo       ! H2
  REAL (KIND=DP), PUBLIC                              :: oldhd        ! HD
  REAL (KIND=DP), PUBLIC                              :: oldco        ! CO
  REAL (KIND=DP), PUBLIC                              :: oldc         ! C
END MODULE CONVER

MODULE PROFIL

! Cloud profile related data

  use constantes
  use STR_DATA
  use CONVER

!  Variables used in numerical algorithms (sort of...)

  INTEGER, PUBLIC                                     :: isoneside    ! 1 if one side, 0 if two sides
  INTEGER, PUBLIC                                     :: ifafm =  20  ! Number of global iteration on whole structure
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tau_imp      ! Mandatory points in optical depth grid
  INTEGER, PUBLIC                                     :: is_imp = 0   ! 1: we are currently computing a mandatory point
  INTEGER, PUBLIC                                     :: next_imp = 1 ! next mandatory point

!  Model defining variables

  CHARACTER (LEN=35), PUBLIC                          :: modele       ! Root name of output files
  CHARACTER (LEN=35), PUBLIC                          :: chimie       ! Name of chemistry file
  CHARACTER (LEN=35), PUBLIC                          :: los_ext      ! Line of sight extinction curve
  CHARACTER (LEN=35), PUBLIC                          :: srcpp        ! Radiation source (star)
  REAL (KIND=DP), PUBLIC                              :: d_sour       ! Distance of radiation source (0.0 -> infinity)
  REAL (KIND=DP), PUBLIC                              :: rstar        ! Radius of emitting source (read in rsun, converted to cm)
  REAL (KIND=DP), PUBLIC                              :: teff         ! Effective temperature (K)
  REAL (KIND=DP), PUBLIC                              :: taumax       ! Maximal optical depth computed
  REAL (KIND=DP), PUBLIC                              :: Avmax        ! Maximal AV computed (taumax * 2.5 log10(e))
  REAL (KIND=DP), PUBLIC                              :: opdinf       ! Optical depth for 1 sided clouds
  REAL (KIND=DP), PUBLIC                              :: nh_init      ! Initial density
  REAL (KIND=DP), PUBLIC                              :: tg_init      ! Initial temperature
  REAL (KIND=DP), PUBLIC                              :: presse       ! External pressure (if ifisob = 2)
  REAL (KIND=DP), PUBLIC                              :: fmrc         ! Cosmic rays inisation scaling
  REAL (KIND=DP), PUBLIC                              :: zeta         ! Cosmic rays ionisatin rate (zeta = 1.0e-17 * fmrc)
  REAL (KIND=DP), PUBLIC                              :: fphsec       ! Secondary photons flux
  REAL (KIND=DP), PUBLIC                              :: vturb        ! Turbulent velocity (Doppler broadening only)
  REAL (KIND=DP), PUBLIC                              :: radm         ! Radiation field intensity on left side
  REAL (KIND=DP), PUBLIC                              :: radp         ! Radiation field intensity on right side
  REAL (KIND=DP), PUBLIC                              :: radm0        ! Radiation field intensity at infinity on left side
  REAL (KIND=DP), PUBLIC                              :: radp0        ! Radiation field intensity at infinity on right side
  REAL (KIND=DP), PUBLIC                              :: cdunit       ! Av to gaz Column Density convertion
  REAL (KIND=DP), PUBLIC                              :: rrr          ! Ratio Av / E(B-V)
  REAL (KIND=DP), PUBLIC                              :: alb          ! Dust albedo
  REAL (KIND=DP), PUBLIC                              :: gg           ! Anisotropy (<cos(theta)>)
  REAL (KIND=DP), PUBLIC                              :: gratio       ! Grain/gaz mass ratio
  REAL (KIND=DP), PUBLIC                              :: rhogr        ! Grain density
  REAL (KIND=DP), PUBLIC                              :: alpgr        ! Grain size distribution index
  REAL (KIND=DP), PUBLIC                              :: rgrmin       ! Minimum grain size
  REAL (KIND=DP), PUBLIC                              :: rgrmax       ! Maximum grain size
  REAL (KIND=DP), PUBLIC                              :: dsite        ! Mean distance between sites on grains
  REAL (KIND=DP), PUBLIC                              :: cc1, cc2, cc3  !  Fitzpatrick fit parameters
  REAL (KIND=DP), PUBLIC                              :: cc4, xgam, y0  !  Fitzpatrick fit parameters
  INTEGER, PUBLIC                                     :: ifeqth       ! 1 : Temperature from thermal balance
  INTEGER, PUBLIC                                     :: ifisob       ! 2 : Isobaric state equation
  INTEGER, PUBLIC                                     :: l0h2         ! Highest H2 level included
  INTEGER, PUBLIC                                     :: ichh2        ! H + H2 collisions
  INTEGER, PUBLIC                                     :: l0hd         ! Highest HD level included
  INTEGER, PUBLIC                                     :: itrh2        ! Shall we include H2 photdissociation ?
  INTEGER, PUBLIC                                     :: iforh2       ! H2 formation mechanism
  INTEGER, PUBLIC                                     :: istic        ! H Sticking coefficient
  INTEGER, PUBLIC                                     :: jlyman = 0   ! Include (1) or not (0) H Lyman lines
  INTEGER, PUBLIC                                     :: jfgkh2 = 0   ! Lowest J treated by FGK approximation
  INTEGER, PUBLIC                                     :: itrhd        ! Shall we include HD photdissociation ?
  INTEGER, PUBLIC                                     :: mjhd         ! Upper J level of HD included
  INTEGER, PUBLIC                                     :: itrco0       ! Shall we include CO0 photdissociation ?
  INTEGER, PUBLIC                                     :: itrco1       ! Shall we include CO1 photdissociation ?
  INTEGER, PUBLIC                                     :: itrco2       ! Shall we include CO2 photdissociation ?
  INTEGER, PUBLIC                                     :: itrco3       ! Shall we include CO3 photdissociation ?
  INTEGER, PUBLIC                                     :: ifrac        ! Number of slice in fractal grid
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tfrac        ! Tau grid for fractal model
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: nfrac        ! nH grid for fractal model
  INTEGER, PUBLIC                                     :: pr_np        ! Number of points in interpolation grid
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: pr_tau       ! Tau grid for interpolation
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: pr_tem       ! Temperature grid for interpolation
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: pr_den       ! Density grid for interpolation

!  Variables used for current iteration

  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: tau          ! Optical depth profile
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: densh        ! Density profile
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: tgaz         ! Temperature profile
  REAL (KIND=DP), PUBLIC, DIMENSION(0:nes,0:noptm)    :: abnua        ! species abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(nlevh2,0:noptm)   :: xreh2        ! H2 levels relative abundances (X level)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:nvbc,0:njbc,0:noptm) :: h2bnua  ! H2 populations (B level)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:nvbc,0:njbc,0:noptm) :: h2cnua  ! H2 populations (C level)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timh21       ! Formation and destruction of H2
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timh22       ! Formation and destruction of H2
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timh23       ! Formation and destruction of H2
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timh24       ! Formation and destruction of H2
  REAL (KIND=DP), PUBLIC, DIMENSION(npmg,0:noptm)     :: t_25         ! Formation of H2 on grains
  REAL (KIND=DP), PUBLIC, DIMENSION(nlevhd,0:noptm)   :: xrehd        ! HD levels relative abundances
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timhd1       ! Formation and destruction of HD
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timhd2       ! Formation and destruction of HD
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timhd3       ! Formation and destruction of HD
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timhd4       ! Formation and destruction of HD
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timco1       ! Formation and destruction of CO
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timco2       ! Formation and destruction of CO
  REAL (KIND=DP), PUBLIC, DIMENSION(0:noptm)          :: timco3       ! Formation and destruction of CO

  REAL (KIND=DP), PUBLIC, DIMENSION(nlevh2,0:noptm)   :: cdth2        ! H2 weighted column density (by T)
  REAL (KIND=DP), PUBLIC, DIMENSION(nlevhd,0:noptm)   :: cdthd        ! HD weighted column density (by T)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:njco,0:noptm)   :: cdtc0        ! CO0 weighted column density (by T)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:njco,0:noptm)   :: cdtc1        ! CO1 weighted column density (by T)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:njco,0:noptm)   :: cdtc2        ! CO2 weighted column density (by T)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:njco,0:noptm)   :: cdtc3        ! CO3 weighted column density (by T)

  REAL (KIND=DP), PUBLIC, DIMENSION(0:np)             :: ab           ! abundances at current point (temporary variable)
  REAL (KIND=DP), PUBLIC, DIMENSION(0:miter)          :: tb           ! temperature iteration at point iopt

!  Variables saved from previous iteration

  INTEGER, PUBLIC                                     :: npo          ! Number of points in previous grid
  INTEGER, PUBLIC                                     :: n_imp        ! Number of imposed point in Av
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tau_o        ! previous optical depth profile (dust at V)
  REAL (KIND=DP)                                      :: abhm
  REAL (KIND=DP)                                      :: abhp
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: abh_o        ! previous atomic hydrogen profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: abh2_o       ! previous molecular hydrogen profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: xh2_o        ! previous H2(J) fractional abundance
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: tgaz_o       ! previous temperature profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:), ALLOCATABLE   :: dens_o       ! previous density profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdth2_o      ! previous integrated weighted H2 profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdthd_o      ! previous integrated weighted HD profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc0_o      ! previous integrated weighted CO0 profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc1_o      ! previous integrated weighted CO1 profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc2_o      ! previous integrated weighted CO2 profile
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: cdtc3_o      ! previous integrated weighted CO3 profile

  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdcat_o     ! previous optical depth in lines of C
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdnat_o     ! previous optical depth in lines of N
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdoat_o     ! previous optical depth in lines of O
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdsat_o     ! previous optical depth in lines of S
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdsiat_o    ! previous optical depth in lines of Si
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdcpl_o     ! previous optical depth in lines of C+
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdnpl_o     ! previous optical depth in lines of N+
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdopl_o     ! previous optical depth in lines of O+
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdspl_o     ! previous optical depth in lines of S+
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdsip_o     ! previous optical depth in lines of Si+
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco0_o     ! previous optical depth in lines of CO0
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco1_o     ! previous optical depth in lines of CO1
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco2_o     ! previous optical depth in lines of CO2
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdco3_o     ! previous optical depth in lines of CO3
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdcs0_o     ! previous optical depth in lines of CS0
  REAL (KIND=DP), PUBLIC, DIMENSION(:,:), ALLOCATABLE :: opdhcop_o    ! previous optical depth in lines of HCO+

  REAL (KIND=DP), PUBLIC, DIMENSION(ntrcat,0:noptm)   :: opdcat       ! optical depth in lines of C
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrnat,0:noptm)   :: opdnat       ! optical depth in lines of N
  REAL (KIND=DP), PUBLIC, DIMENSION(ntroat,0:noptm)   :: opdoat       ! optical depth in lines of O
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrsat,0:noptm)   :: opdsat       ! optical depth in lines of S
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrsiat,0:noptm)  :: opdsiat      ! optical depth in lines of Si
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrcpl,0:noptm)   :: opdcpl       ! optical depth in lines of C+
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrnpl,0:noptm)   :: opdnpl       ! optical depth in lines of S+
  REAL (KIND=DP), PUBLIC, DIMENSION(ntropl,0:noptm)   :: opdopl       ! optical depth in lines of O+
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrspl,0:noptm)   :: opdspl       ! optical depth in lines of S+
  REAL (KIND=DP), PUBLIC, DIMENSION(ntrsip,0:noptm)   :: opdsip       ! optical depth in lines of Si+
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)     :: opdco0       ! optical depth in lines of CO0
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)     :: opdco1       ! optical depth in lines of CO1
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)     :: opdco2       ! optical depth in lines of CO2
  REAL (KIND=DP), PUBLIC, DIMENSION(njco,0:noptm)     :: opdco3       ! optical depth in lines of CO3
  REAL (KIND=DP), PUBLIC, DIMENSION(njcs,0:noptm)     :: opdcs0       ! optical depth in lines of CS0
  REAL (KIND=DP), PUBLIC, DIMENSION(njhcop,0:noptm)   :: opdhcop      ! optical depth in lines of HCO+


CONTAINS

  subroutine INIT_PROFIL

    use constantes
    use inoutput
    USE STR_DATA

    implicit none

    REAL (KIND=DP)     :: pas = 1.0e-7_DP, t, cte
    REAL (KIND=DP)     :: tauc
    INTEGER, PARAMETER :: n_obl = 8
    REAL (KIND=DP), DIMENSION(n_obl+1) :: pt_obl
    INTEGER            :: npp, i
    INTEGER            :: i_obl, j_obl

    data pt_obl / 0.2_dp, 0.3_dp, 0.5_dp, 0.7_dp &
                , 2.0_dp, 3.0_dp, 5.0_dp, 7.0_dp, 1.0e10_dp /

!  Control of variable step size (in tau)

       if (ifisob == 5) then
         varmax = 1.0_dp
         dtaumin = 1.0e-3_DP
       else
         varmax = 0.5_dp
         dtaumin = 1.0e-2_DP
       endif

    pt_obl = pt_obl / tautav

!  If l0h2 < 40, it is not possible to have 1/3 of H2 formation
!     energy in internal energy

    if (iforh2 == 0 .AND. l0h2 < 40) then
      print *, "   WARNING !: iforh2 changed to 1 (not enough internal energy)"
      iforh2 = 1
    endif

!  If radp = 0.0 then radiation field is on one side only
!     and there is no need to iterate. Else, iteration is mandatory

    radm0 = radm
    radp0 = radp

    if (radp == 0.0_DP) then
      isoneside = 1
    else
      isoneside = 0
    endif

    if (isoneside == 1) then
      opdinf = 1.0e99_dp
    else
      opdinf = 0.0e0_dp
    endif

    taumax = Avmax / tautav
    pas = pas / tautav
    tau(0) = zero
    tau(1) = pas * 0.5_DP

!  30 mai2002, grille en temperature et densite pour interpolation lineaire

!   if (abs(ifisob) == 1 .OR. abs(ifisob) == 2) then
    if (abs(ifisob) == 1) then
      fichier = TRIM(data_dir)//TRIM(modele)//'.pfl'
      open (66, file=fichier, status="old")
      read (66,*) pr_np
      if (ifisob < 0) then
        ALLOCATE (pr_tau(0:2*pr_np))
        ALLOCATE (pr_tem(0:2*pr_np))
        ALLOCATE (pr_den(0:2*pr_np))
      else
        ALLOCATE (pr_tau(0:pr_np))
        ALLOCATE (pr_tem(0:pr_np))
        ALLOCATE (pr_den(0:pr_np))
      endif
      do i=0,pr_np
        read (66,*) pr_tau(i), pr_tem(i), pr_den(i)
        pr_tau(i) = pr_tau(i) / tautav
      end do
      nh_init = pr_den(0)
      tg_init = pr_tem(0)
      close (66)
      if (ifisob < 0) then
        do i=0,pr_np-1
          pr_tau(2*pr_np-i) = 2.0_DP * pr_tau(pr_np) - pr_tau(i)
          pr_tem(2*pr_np-i) = pr_tem(i)
          pr_den(2*pr_np-i) = pr_den(i)
        end do
        ifisob = -ifisob
        pr_np = pr_np * 2
      endif

    endif

!  3 mai 2002, grille pour un modele fractal

    if (ifisob == 5) then

      print *, "  Warning : ifisob =", ifisob

      fichier = TRIM(data_dir)//TRIM(modele)//'.pfl'
      open (66, file=fichier, status="old")
      read (66,*) ifrac
      ALLOCATE (tfrac(0:ifrac))
      ALLOCATE (nfrac(0:ifrac))
      do i=0,ifrac
        read (66,*) tfrac(i), nfrac(i)
        tfrac(i) = taumax * tfrac(i)
        nfrac(i) = nfrac(i) * nh_init
      end do
      close (66)

      if (isoneside == 1) then

        npp = int(log10(tfrac(1)/pas)) + 1
        npo = npp + ifrac
        ALLOCATE(tau_imp(0:npo))
        n_imp = npo
        tau_imp(0) = 0.0_DP
        do i=1,npp
          tau_imp(i) = pas
          pas = pas * 10.0_DP
        end do
        tau_imp(npp+1:npp+ifrac) = tfrac(1:ifrac)
        do i=1,npo
          print *, i, tau_imp(i), tau_imp(i) * tautav
        end do

      else

        npp = int(log10(tfrac(1)/pas))
        npo = 2 * npp + ifrac
        ALLOCATE(tau_imp(0:npo))
        n_imp = npo
        tau_imp(0) = 0.0_DP
        do i=1,npp
          tau_imp(i) = pas
          tau_imp(npo-i) = taumax - pas
          pas = pas * 10.0_DP
        end do
        tau_imp(npp+1:npp+ifrac-1) = tfrac(1:ifrac-1)
        tau_imp(npo) = taumax
        do i=1,npo
          print *, i, tau_imp(i), tau_imp(i) * tautav
        end do
      endif
    else
!  Construction des points "obliges"
!  8 oct 2003 : same points whether or not we are "two-sided"

      tauc = 0.5_dp * taumax
      npp = int(log10(tauc/pas)) + 1
      j_obl = 0
      do i=1,n_obl
        if (pt_obl(i) < tauc) then
          j_obl = j_obl + 1
        endif
      end do
      npo = 2 * (npp + j_obl) + 2
      ALLOCATE(tau_imp(0:npo))
      n_imp = npo
      tau_imp(0) = 0.0_DP
      i_obl = 1
      i = 1
      do
        if (pas < pt_obl(i_obl)) then
          tau_imp(i) = pas
          tau_imp(npo-i) = taumax - pas
          pas = pas * 10.0_DP
        else
          tau_imp(i) = pt_obl(i_obl)
          tau_imp(npo-i) = taumax - pt_obl(i_obl)
          i_obl = i_obl + 1
        endif
        i = i + 1
        if (i == npp + j_obl + 1) then
          exit
        endif
      end do
      tau_imp(npp+j_obl+1) = tauc
      tau_imp(npo) = taumax
      do i=1,npo
        print *, i, tau_imp(i), tau_imp(i) * tautav
      end do
    endif

!  Initial integrated weighted molecular profiles and optical depth

    ALLOCATE (tau_o(0:npo))
    ALLOCATE (abh_o(0:npo))
    ALLOCATE (abh2_o(0:npo))
    if (jfgkh2 > 0) then
      ALLOCATE (xh2_o(0:jfgkh2-1,0:npo))
    endif
    ALLOCATE (tgaz_o(0:npo))
    ALLOCATE (dens_o(0:npo))
    ALLOCATE (cdth2_o(nlevh2,0:npo))
    ALLOCATE (cdthd_o(nlevhd,0:npo))
    ALLOCATE (cdtc0_o(0:njco,0:npo))
    ALLOCATE (cdtc1_o(0:njco,0:npo))
    ALLOCATE (cdtc2_o(0:njco,0:npo))
    ALLOCATE (cdtc3_o(0:njco,0:npo))

    ALLOCATE (opdcat_o(ntrcat,0:npo))
    ALLOCATE (opdnat_o(ntrnat,0:npo))
    ALLOCATE (opdoat_o(ntroat,0:npo))
    ALLOCATE (opdsat_o(ntrsat,0:npo))
    ALLOCATE (opdsiat_o(ntrsiat,0:npo))
    ALLOCATE (opdcpl_o(ntrcpl,0:npo))
    ALLOCATE (opdnpl_o(ntrnpl,0:npo))
    ALLOCATE (opdopl_o(ntropl,0:npo))
    ALLOCATE (opdspl_o(ntrspl,0:npo))
    ALLOCATE (opdsip_o(ntrsip,0:npo))
    ALLOCATE (opdco0_o(njco,0:npo))
    ALLOCATE (opdco1_o(njco,0:npo))
    ALLOCATE (opdco2_o(njco,0:npo))
    ALLOCATE (opdco3_o(njco,0:npo))
    ALLOCATE (opdcs0_o(njcs,0:npo))
    ALLOCATE (opdhcop_o(njhcop,0:npo))

    tau_o = tau_imp
    do i=0,npo
      if (tau_o(i) == 0.0_dp) then
        abhm = 1.0_dp
      else
        abhm = 0.5_dp - atan(10.0_dp * log10(1.0e4*tau_o(i)/radm)) / xpi
      endif
      if (isoneside == 1) then
        t = tau_o(i)
        abhp = 0.0_dp
      else
        t = min(tau_o(i),taumax-tau_o(i))
        if (t == 0.0_dp) then
          abhp = 1.0_dp
        else
          abhp = 0.5_dp - atan(10.0_dp &
               * log10(1.0e4*(taumax-tau_o(i))/radp)) / xpi
        endif
      endif
!     abh_o(i) = nh_init * exp(-5.0_dp*t) + 1.0_dp
      abh_o(i) = nh_init * max(abhm, abhp)
      abh2_o(i) = (nh_init - abh_o(i)) * 0.5_dp + 1.0_dp
      tgaz_o(i) = tg_init * exp(-3.0_dp*t) + 10.0_dp
    end do
    if (jfgkh2 > 0) then
      xh2_o(0,:) = 0.25_dp
    endif
    if (jfgkh2 > 1) then
      xh2_o(1,:) = 0.75_dp
    endif
    if (jfgkh2 > 2) then
      do i=2,jfgkh2-1
        xh2_o(i,:) = dix**(-i)
      end do
    endif
    dens_o = nh_init

    opdcat_o = SPREAD(tau_imp, 1, ntrcat)
    opdnat_o = SPREAD(tau_imp, 1, ntrnat)
    opdoat_o = SPREAD(tau_imp, 1, ntroat)
    opdsat_o = SPREAD(tau_imp, 1, ntrsat)
    opdsiat_o = SPREAD(tau_imp, 1, ntrsiat)
    opdcpl_o = SPREAD(tau_imp, 1, ntrcpl)
    opdnpl_o = SPREAD(tau_imp, 1, ntrnpl)
    opdopl_o = SPREAD(tau_imp, 1, ntropl)
    opdspl_o = SPREAD(tau_imp, 1, ntrspl)
    opdsip_o = SPREAD(tau_imp, 1, ntrsip)
    opdco0_o = SPREAD(tau_imp, 1, njco)
    opdco1_o = SPREAD(tau_imp, 1, njco)
    opdco2_o = SPREAD(tau_imp, 1, njco)
    opdco3_o = SPREAD(tau_imp, 1, njco)
    opdcs0_o = SPREAD(tau_imp, 1, njcs)
    opdhcop_o = SPREAD(tau_imp, 1, njhcop)

!  Initialisation of various other profiles

    if (ifisob == 5) then
      densh = nfrac(1)
    else
      densh = nh_init
    endif
    tgaz  = tg_init
    zeta = 1.0e-17_DP * fmrc

!  flux de photons secondaires due aux rayons cosmiques:
!  1E3 cm-2 s-1 (Roberge, in Dalgarno's book)

   fphsec = 1.0e3_DP * fmrc

!  conversion of turbulent velocity from km s-1 to cm s-1

   vturb = vturb * 1.0e5_DP

!  Extinction curve

   call FM_PAR

  END subroutine INIT_PROFIL

!      %%%%%%%%%%%%%%%%
       subroutine REALL
!      %%%%%%%%%%%%%%%%

       use constantes
       use STR_DATA
       use SP_DATA

       implicit none

       integer :: i

       DEALLOCATE (tau_o)
       DEALLOCATE (abh_o)
       DEALLOCATE (abh2_o)
       if (jfgkh2 > 0) then
         DEALLOCATE (xh2_o)
       endif
       DEALLOCATE (tgaz_o)
       DEALLOCATE (dens_o)
       DEALLOCATE (cdth2_o)
       DEALLOCATE (cdthd_o)
       DEALLOCATE (cdtc0_o)
       DEALLOCATE (cdtc1_o)
       DEALLOCATE (cdtc2_o)
       DEALLOCATE (cdtc3_o)

       DEALLOCATE (opdcat_o)
       DEALLOCATE (opdnat_o)
       DEALLOCATE (opdoat_o)
       DEALLOCATE (opdsat_o)
       DEALLOCATE (opdsiat_o)
       DEALLOCATE (opdcpl_o)
       DEALLOCATE (opdnpl_o)
       DEALLOCATE (opdopl_o)
       DEALLOCATE (opdspl_o)
       DEALLOCATE (opdsip_o)
       DEALLOCATE (opdco0_o)
       DEALLOCATE (opdco1_o)
       DEALLOCATE (opdco2_o)
       DEALLOCATE (opdco3_o)
       DEALLOCATE (opdcs0_o)
       DEALLOCATE (opdhcop_o)

       ALLOCATE (tau_o(0:npo))
       ALLOCATE (abh_o(0:npo))
       ALLOCATE (abh2_o(0:npo))
       if (jfgkh2 > 0) then
         ALLOCATE (xh2_o(0:jfgkh2-1,0:npo))
       endif
       ALLOCATE (tgaz_o(0:npo))
       ALLOCATE (dens_o(0:npo))
       ALLOCATE (cdth2_o(nlevh2,0:npo))
       ALLOCATE (cdthd_o(nlevhd,0:npo))
       ALLOCATE (cdtc0_o(0:njco,0:npo))
       ALLOCATE (cdtc1_o(0:njco,0:npo))
       ALLOCATE (cdtc2_o(0:njco,0:npo))
       ALLOCATE (cdtc3_o(0:njco,0:npo))

       ALLOCATE (opdcat_o(ntrcat,0:npo))
       ALLOCATE (opdnat_o(ntrnat,0:npo))
       ALLOCATE (opdoat_o(ntroat,0:npo))
       ALLOCATE (opdsat_o(ntrsat,0:npo))
       ALLOCATE (opdsiat_o(ntrsiat,0:npo))
       ALLOCATE (opdcpl_o(ntrcpl,0:npo))
       ALLOCATE (opdnpl_o(ntrnpl,0:npo))
       ALLOCATE (opdopl_o(ntropl,0:npo))
       ALLOCATE (opdspl_o(ntrspl,0:npo))
       ALLOCATE (opdsip_o(ntrsip,0:npo))
       ALLOCATE (opdco0_o(njco,0:npo))
       ALLOCATE (opdco1_o(njco,0:npo))
       ALLOCATE (opdco2_o(njco,0:npo))
       ALLOCATE (opdco3_o(njco,0:npo))
       ALLOCATE (opdcs0_o(njcs,0:npo))
       ALLOCATE (opdhcop_o(njhcop,0:npo))

       call CDT

       tau_o = tau(0:npo)
       dens_o = densh(0:npo)
       tgaz_o = tgaz(0:npo)
       abh_o = abnua(i_h,0:npo)
       abh2_o = abnua(i_h2,0:npo)
       do i=0,jfgkh2-1
         xh2_o(i,:) = xreh2(i+1,0:npo)
       end do
       cdth2_o = cdth2(:,0:npo)
       cdthd_o = cdthd(:,0:npo)
       cdtc0_o = cdtc0(:,0:npo)
       cdtc1_o = cdtc1(:,0:npo)
       cdtc2_o = cdtc2(:,0:npo)
       cdtc3_o = cdtc3(:,0:npo)

       opdcat_o = opdcat(:,0:npo)
       opdnat_o = opdnat(:,0:npo)
       opdoat_o = opdoat(:,0:npo)
       opdsat_o = opdsat(:,0:npo)
       opdsiat_o = opdsiat(:,0:npo)
       opdcpl_o = opdcpl(:,0:npo)
       opdnpl_o = opdnpl(:,0:npo)
       opdopl_o = opdopl(:,0:npo)
       opdspl_o = opdspl(:,0:npo)
       opdsip_o = opdsip(:,0:npo)
       opdco0_o = opdco0(:,0:npo)
       opdco1_o = opdco1(:,0:npo)
       opdco2_o = opdco2(:,0:npo)
       opdco3_o = opdco3(:,0:npo)
       opdcs0_o = opdcs0(:,0:npo)
       opdhcop_o = opdhcop(:,0:npo)

       return
       end subroutine REALL

!      %%%%%%%%%%%%%%
       subroutine CDT
!      %%%%%%%%%%%%%%

       use constantes
!      use auxiliaries
!      USE RF_TOOLS
       USE SP_DATA
       USE H2_DATA
       USE HD_DATA
       USE STR_DATA

        implicit none

        REAL (KIND=DP)  :: ct2, ct3, deltau
        REAL (KIND=DP)  :: b0sb_p, b0sb_i, vt2, cden
        INTEGER         :: iopt, jopt, lev, j

!     *********************************************************************
!               vturb donne en cm s-1
!               ct2= 2k/mua   = 1.662902e8  en cgs
!               bdop:  dimension vitesse cm s-1 a
!      *****************************************************************

!  CDUNIT proportion gaz poussiere: NH = CDUNIT * E(B-V)
!  RRR proportion indice de couleur extinction: Av = RRR * E(B-V)

    ct2 = deux * xk / amu
    ct3 = cdunit * tautav / rrr
    vt2 = vturb * vturb

    do iopt=0,npo

       if (iopt > 0) then
         jopt = iopt - 1
       else
         jopt = 0
       endif
         deltau = tau(iopt) - tau(jopt)

!  calcul de la colonne densite de h2 pour chaque niveau
!    ponderee par la largeur doppler

       b0sb_p = vturb / dsqrt(vt2 + ct2 * tgaz(jopt) / amh2)
       b0sb_i = vturb / dsqrt(vt2 + ct2 * tgaz(iopt) / amh2)

       do lev=1,l0h2
         cden = ct3 * deltau * 0.5_DP &
              * ((abnua(i_h2,iopt) * xreh2(lev,iopt) * b0sb_i / densh(iopt)) &
              + (abnua(i_h2,jopt) * xreh2(lev,jopt) * b0sb_p / densh(jopt)))
         if (cden < 0.0_dp) then
           print *, "CDH2", lev, jopt, xreh2(lev,jopt)
           print *, "CDH2", lev, iopt, xreh2(lev,iopt)
!          stop
           cden = 0.0_dp
         endif
         cdth2(lev,iopt) = cdth2(lev,jopt) + cden
       end do

!  calcul de la colonne densite de hd pour chaque niveau
!    ponderee par la largeur doppler

       if (itrhd .ne. 0) then
         b0sb_p = vturb / dsqrt(vt2 + ct2 * tgaz(jopt) / amhd)
         b0sb_i = vturb / dsqrt(vt2 + ct2 * tgaz(iopt) / amhd)

         do lev=1,l0hd
           cden = ct3 * deltau * 0.5_DP &
                * ((abnua(i_hd,iopt) * xrehd(lev,iopt) * b0sb_i / densh(iopt)) &
                + (abnua(i_hd,jopt) * xrehd(lev,jopt) * b0sb_p / densh(jopt)))
           cdthd(lev,iopt) = cdthd(lev,jopt) + cden
         end do
       endif

!  calcul de la colonne densite de co pour chaque j

       b0sb_p = vturb / dsqrt(vt2 + ct2 * tgaz(jopt) / amco0)
       b0sb_i = vturb / dsqrt(vt2 + ct2 * tgaz(iopt) / amco0)

       do j=0,njco
         cden = ct3 * deltau * 0.5_DP &
          * ((abnua(i_co0,iopt) * xreco0(j+1,iopt) * b0sb_i / densh(iopt)) &
          + (abnua(i_co0,jopt) * xreco0(j+1,jopt) * b0sb_p / densh(jopt)))
         cdtc0(j,iopt) = cdtc0(j,jopt) + cden
       end do

!  calcul de la colonne densite de co1 pour chaque j

       if (i_c13 .ne. 0) then

         b0sb_p = vturb / dsqrt(vt2 + ct2 * tgaz(jopt) / amco1)
         b0sb_i = vturb / dsqrt(vt2 + ct2 * tgaz(iopt) / amco1)

         do j=0,njco
           cden = ct3 * deltau * 0.5_DP &
             * ((abnua(i_co1,iopt) * xreco1(j+1,iopt) * b0sb_i / densh(iopt)) &
             + (abnua(i_co1,jopt) * xreco1(j+1,jopt) * b0sb_p / densh(jopt)))
           cdtc1(j,iopt) = cdtc1(j,jopt) + cden
         end do

       endif

!  calcul de la colonne densite de co2 pour chaque j

       if (i_o18 .ne. 0) then

         b0sb_p = vturb / dsqrt(vt2 + ct2 * tgaz(jopt) / amco2)
         b0sb_i = vturb / dsqrt(vt2 + ct2 * tgaz(iopt) / amco2)

         do j=0,njco
           cden = ct3 * deltau * 0.5_DP &
             * ((abnua(i_co2,iopt) * xreco2(j+1,iopt) * b0sb_i / densh(iopt)) &
             + (abnua(i_co2,jopt) * xreco2(j+1,jopt) * b0sb_p / densh(jopt)))
           cdtc2(j,iopt) = cdtc2(j,jopt) + cden
         end do

       endif

!  calcul de la colonne densite de co3 pour chaque j

       if (i_c13 .ne. 0 .and. i_o18 .ne. 0) then

         b0sb_p = vturb / dsqrt(vt2 + ct2 * tgaz(jopt) / amco3)
         b0sb_i = vturb / dsqrt(vt2 + ct2 * tgaz(iopt) / amco3)

         do j=0,njco
           cden = ct3 * deltau * 0.5_DP &
             * ((abnua(i_co3,iopt) * xreco3(j+1,iopt) * b0sb_i / densh(iopt)) &
             + (abnua(i_co3,jopt) * xreco3(j+1,jopt) * b0sb_p / densh(jopt)))
           cdtc3(j,iopt) = cdtc3(j,jopt) + cden
         end do

       endif

    end do

    return
    end subroutine CDT

!   %%%%%%%%%%%%%%%%%
    subroutine FM_PAR
!   %%%%%%%%%%%%%%%%%

!  Fitzpatrick and Massa data base of extinction curves

    print *, los_ext

    select case (TRIM(ADJUSTL(los_ext)))
      case ("Galaxy")
!  reference : Seaton galactic average

        cc1 = -0.38_DP
        cc2 = 0.74_DP
        cc3 = 3.96_DP
        cc4 = 0.26_DP
        xgam = 1.05_DP
        y0 = 4.59_DP

        if (rrr /= 3.1_DP) then
          print *, "  Warning : Standard Rv is 3.1"
          print *, "            yours is :", rrr
        end if

      case ("LMC")
!  reference : FM , ApJ 328, 734 (1988)

        cc1 = -0.687_DP
        cc2 = 0.891_DP
        cc3 = 2.550_DP
        cc4 = 0.504_DP
        xgam = 0.994_DP
        y0 = 4.608_DP

!  Rv : Weingartner & Draine (2001)
        if (rrr /= 2.6_DP) then
          print *, "  Warning : Standard Rv is 2.6"
          print *, "            yours is :", rrr
        end if

      case ("SMC")
!  reference : ???

        cc1 = -4.0_DP
        cc2 = 2.0_DP
        cc3 = 0.0_DP
        cc4 = 0.85_DP
        xgam = 0.0_DP
        y0 = 0.0_DP

!  Rv : Weingartner & Draine (2001)
        if (rrr /= 2.87_DP) then
          print *, "  Warning : Standard Rv is 2.87"
          print *, "            yours is :", rrr
        end if

      case ("Orion_Bar")
!  reference : theta1 Ori C 
!  reference : FM , ApJ 328, 734 (1988)
!  + CCM, ApJ345, 245 (1989)

        cc1 = 1.251_DP
        cc2 = 0.033_DP
        cc3 = 1.331_DP
        cc4 = 0.186_DP
        xgam = 0.846_DP
        y0 = 4.635_DP

        if (rrr /= 5.5_DP) then
          print *, "  Warning : Standard Rv is 5.5"
          print *, "            yours is :", rrr
        end if

      case ("HD37903")
!  reference : FM , ApJ 328, 734 (1988)
!  + CCM, ApJ345, 245 (1989)

        cc1 = 0.965_DP
        cc2 = 0.384_DP
        cc3 = 3.300_DP
        cc4 = 0.440_DP
        xgam = 1.045_DP
        y0 = 4.615_DP

        if (rrr /= 4.11_DP) then
          print *, "  Warning : Standard Rv is 4.11"
          print *, "            yours is :", rrr
        end if


      case ("HD27778")
!  reference : Fitzpatrick, private communication

        cc1 = -0.433_DP
        cc2 = 0.77_DP
        cc3 = 3.44_DP
        cc4 = 0.745_DP
        xgam = 1.136_DP
        y0 = 4.578_DP

! RV : (Rachford 2002)
        if (rrr /= 2.65_DP) then
          print *, "  Warning : Standard Rv is 2.65"
          print *, "            yours is :", rrr
        end if

      case ("HD34078")
!  reference : Fitzpatrick et Massa 1990 (AE Aur)

        cc1 = 0.473_DP
        cc2 = 0.571_DP
        cc3 = 4.152_DP
        cc4 = 0.52_DP
        xgam = 1.087_DP
        y0 = 4.589_DP

! Rv : (Patriarchi et al. 2001)
        if (rrr /= 3.42_DP) then
          print *, "  Warning : Standard Rv is 3.42"
          print *, "            yours is :", rrr
        end if

      case ("HD73882")
!  reference : Fitzpatrick et Massa 1990

        cc1 = -0.412_DP
        cc2 = 0.788_DP
        cc3 = 3.342_DP
        cc4 = 0.54_DP
        xgam = 1.192_DP
        y0 = 4.576_DP

! Rv : Rachford et al. 2002
        if (rrr /= 3.39_DP) then
          print *, "  Warning : Standard Rv is 3.39"
          print *, "            yours is :", rrr
        end if

      case ("HD102065")
!  reference : ???

        cc1 = -0.48_DP
        cc2 = 0.58_DP
        cc3 = 8.21_DP
        cc4 = -0.01_DP
        xgam = 1.18_DP
        y0 = 4.58_DP

        if (rrr /= 3.9_DP) then
          print *, "  Warning : Standard Rv is 3.9"
          print *, "            yours is :", rrr
        end if

      case ("HD96675")
!  reference : ???

        cc1 = 0.99_DP
        cc2 = 0.35_DP
        cc3 = 4.31_DP
        cc4 = 0.58_DP
        xgam = 1.10_DP
        y0 = 4.63_DP

        if (rrr /= 3.5_DP) then
          print *, "  Warning : Standard Rv is 3.9"
          print *, "            yours is :", rrr
        end if

      case ("HD147889")
!  reference : FM , ApJ 328, 734 (1988)
!  + CCM, ApJ345, 245 (1989)

        cc1 = 1.449_DP
        cc2 = 0.151_DP
        cc3 = 4.472_DP
        cc4 = 0.709_DP
        xgam = 1.082_DP
        y0 = 4.617_DP

        if (rrr /= 4.2_DP) then
          print *, "  Warning : Standard Rv is 4.2"
          print *, "            yours is :", rrr
        end if

      case ("Disk")

        cc1 = 0.025_DP
        cc2 = 0.0_DP
        cc3 = 0.0_DP
        cc4 = 0.0_DP
        xgam = 1.082_DP
        y0 = 4.617_DP

        if (rrr /= 0.31_DP) then
          print *, "  Warning : Standard Rv is 0.31"
          print *, "            yours is :", rrr
        end if

      case DEFAULT

        print *, "  Warning : possible lines of sight are :"
        print *, "      Galaxy      (Rv = 3.1)"
        print *, "      SMC         (Rv = ?)"
        print *, "      Orion_Bar   (Rv = 5.5)"
        print *, "      HD27778     (Rv = 3.1)"
        print *, "      HD34078     (Rv = 3.1)"
        print *, "      HD73882     (Rv = 3.39)"
        print *, "      HD102065    (Rv = 3.9)"
        print *, "      HD147889    (Rv = 4.0)"
        print *, "  In case of doubt, use 'Galaxy'"
        stop

    end select

    return
    end subroutine FM_PAR

END MODULE PROFIL

