
c Copyright (C) 2002 - Observatoire de Paris/Meudon
c                      Universite Paris 7
c                      CNRS (France)
c                      Contact : Jacques.Lebourlot@obspm.fr
c 
c This program is free software; you can redistribute it and/or
c modify it under the terms of the GNU General Public License version 2
c as published by the Free Software Foundation.
c 
c This program is distributed in the hope that it will be useful,
c but WITHOUT ANY WARRANTY; without even the implied warranty of
c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c GNU General Public License for more details.
c 
c You should have received a copy of the GNU General Public License
c along with this program; if not, write to the Free Software
c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       subroutine BILTHM (iopt, itn)
c      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

       use constantes
       use auxiliaries
       USE STR_DATA
       USE PROFIL

       implicit double precision (a-h, o-z)

       real (kind=DP), parameter          :: t_lim_do = tcmb
       real (kind=DP), parameter          :: t_lim_up = 1.0e6_DP

       save

       data eps / 0.9d0 /

c   calcule le bilan thermique et determine la nouvelle temperature
c   les 2 premiers appels constituent une phase d initialisation

c   TB est la meilleure temperature actuelle
c   TA et TC l encadrent avec le bilan thermique calcule a cette iteration
c         (ie l etat actuel de la chimie)
c   TAP et TCP encadrent TB en conservant les valeurs aux iterations precedantes

c   on convient d avoir toujours TA < TB < TC   et TAP < TB < TCP

       if (iopt > 0) then
         oldte = tgaz(iopt-1)
       else
         oldte = tg_init
       endif

       if (itn .eq. 1) then

         ta = temp
         tb(0) = temp
         tc = temp

         call LADENS(temp, iopt)
         call FROID (temp, iopt)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)

         btha = bthb
         bthc = bthb

         tcp = temp * 1.1d0
         tap = dmax1(temp * 0.9d0, t_lim_do)

c   on verifie que l on encadre bien la solution

  10     continue
         if (bthc .gt. zero .and. tc .lt. t_lim_up) then
           ta = tc
           btha = bthc
c          tc = tc * 1.05d0
           tc = tc * 1.02d0
           call LADENS(tc, iopt)
           call FROID (tc, iopt)
           call CHAUD (tc, iopt, itn)
           bthc = chotot(iopt) - reftot(iopt)
           goto 10
         else if (btha .lt. zero .and. ta .gt. t_lim_do) then
           tc = ta
           bthc = btha
c          ta = dmax1(ta * 0.95d0, t_lim_do)
           ta = dmax1(ta * 0.98d0, t_lim_do)
           call LADENS(ta, iopt)
           call FROID (ta, iopt)
           call CHAUD (ta, iopt, itn)
           btha = chotot(iopt) - reftot(iopt)
           goto 10
         endif

         tb(1) = half * (ta + tc)

         temp = tb(1)

         call LADENS(temp, iopt)
         call FROID (temp, iopt)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

         tap = dmin1(tap,ta)
         tcp = dmax1(tcp,tc)

         write(iscrn,*) 
         write(iscrn,*) '--------------- bilan thermique --------------'
         write (iscrn,1001) tap, ta, tb(itn), tc, tcp
     1                  , btha, bthrel, bthc

       else if (itn .ge. 2 .and. itn .le. 5) then

         call LADENS(ta, iopt)
         call FROID (ta, iopt)
         call CHAUD (ta, iopt, itn)
         btha = chotot(iopt) - reftot(iopt)

         call LADENS(tc, iopt)
         call FROID (tc, iopt)
         call CHAUD (tc, iopt, itn)
         bthc = chotot(iopt) - reftot(iopt)

         call LADENS(temp, iopt)
         call FROID (temp, iopt)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)

c   on met a jour les bornes extremes

         if (bthb .gt. zero) then
           tap = dmin1(tap,temp)
         else
           tcp = dmax1(tcp,temp)
         endif

  20     continue
         if (btha*bthc .lt. zero) then
           tb(itn) = ta - btha * (tc - ta) / (bthc - btha)
         else if (btha .lt. zero .and. ta .gt. oldte * 0.7d0
     1             .and. ta .gt. t_lim_do) then
           ta = dmax1(ta * 0.98d0, t_lim_do)
           call LADENS(ta, iopt)
           call FROID (ta, iopt)
           call CHAUD (ta, iopt, itn)
           btha = chotot(iopt) - reftot(iopt)
           goto 20
         else if (bthc .gt. zero .and. tc .lt. oldte * 1.3d0) then
           tc = tc * 1.02d0
           call LADENS(tc, iopt)
           call FROID (tc, iopt)
           call CHAUD (tc, iopt, itn)
           bthc = chotot(iopt) - reftot(iopt)
           goto 20
         else
           tb(itn) = half * (ta + tc)
         endif

         if (tb(itn) .le. t_lim_up .and. tb(itn) .ge. t_lim_do) then
           continue
         else
           if (iopt > 0) then
             tb(itn) = tgaz(iopt-1)
           else
             tb(itn) = tg_init
           endif
         endif

         temp = tb(itn)

         tap = dmin1(tap,ta)
         tcp = dmax1(tcp,tc)

         call LADENS(temp, iopt)
         call FROID (temp, iopt)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

         write(iscrn,*) 
         write(iscrn,*) '--------------- bilan thermique --------------'
         write (iscrn,1001) tap, ta, temp, tc, tcp
     1                  , btha, bthrel, bthc

       else

c   cas general

       call LADENS(ta, iopt)
       call FROID (ta, iopt)
       call CHAUD (ta, iopt, itn)
       btha = chotot(iopt) - reftot(iopt)

       call LADENS(tc, iopt)
       call FROID (tc, iopt)
       call CHAUD (tc, iopt, itn)
       bthc = chotot(iopt) - reftot(iopt)

       call LADENS(temp, iopt)
       call FROID (temp, iopt)
       call CHAUD (temp, iopt, itn)
       bthb = chotot(iopt) - reftot(iopt)

c   on met a jour les bornes extremes

       if (bthb .gt. zero) then
         tap = dmax1(tap,temp)
       else
         tcp = dmin1(tcp, temp)
       endif

       if (btha*bthc .lt. zero) then

         if (bthb .gt. zero) then
           ta = temp
           btha = bthb
         else
           tc = temp
           bthc = bthb
         endif

         tb(itn) = ta - btha * (tc - ta) / (bthc - btha)

c  tentatives diverses pour accelerer la convergence

         tb(itn) = tb(itn-1) * (un - eps) + eps * tb(itn)

         if (tb(itn) .gt. tb(itn-1)) then
           if ((tb(itn-2) .gt. tb(itn))
     1         .and. (tb(itn-3) .lt. tb(itn-1))) then
               tb(itn) = half * (tb(itn) + tb(itn-1))
           endif
         else
           if ((tb(itn-2) .lt. tb(itn))
     1         .and. (tb(itn-3) .gt. tb(itn-1))) then
               tb(itn) = half * (tb(itn) + tb(itn-1))
           endif
         endif

         if ((tb(itn) .lt. tb(itn-1))
     1       .and. (tb(itn-1) .lt. tb(itn-2))
     2       .and. (tb(itn-2) .lt. tb(itn-3))) then
           tb(itn) = half * (tb(itn) + tap)
         endif

         if (tb(itn) .le. t_lim_up .and. tb(itn) .ge. t_lim_do) then
           continue
         else
           tb(itn) = tgaz(iopt-1)
         endif

         temp = tb(itn)

         if (ta .gt. temp) then
           ta = temp
         endif
         if (tc .lt. temp) then
           tc = temp
         endif

         call LADENS(ta, iopt)
         call FROID (ta, iopt)
         call CHAUD (ta, iopt, itn)
         btha = chotot(iopt) - reftot(iopt)

         call LADENS(tc, iopt)
         call FROID (tc, iopt)
         call CHAUD (tc, iopt, itn)
         bthc = chotot(iopt) - reftot(iopt)

         call LADENS(temp, iopt)
         call FROID (temp, iopt)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

       else

         tb(itn) = half * (tap + tcp)

         if (tb(itn) .gt. tb(itn-1)) then
           if ((tb(itn-2) .gt. tb(itn))
     1         .and. (tb(itn-3) .lt. tb(itn-1))) then
               tb(itn) = half * (tb(itn) + tb(itn-1))
           endif
         else
           if ((tb(itn-2) .lt. tb(itn))
     1         .and. (tb(itn-3) .gt. tb(itn-1))) then
               tb(itn) = half * (tb(itn) + tb(itn-1))
           endif
         endif

         if ((tb(itn) .lt. tb(itn-1))
     1       .and. (tb(itn-1) .lt. tb(itn-2))
     2       .and. (tb(itn-2) .lt. tb(itn-3))) then
           tb(itn) = half * (tb(itn) + tap)
         endif

         if (tb(itn) .le. t_lim_up .and. tb(itn) .ge. t_lim_do) then
           continue
         else
           if (iopt > 0) then
             tb(itn) = tgaz(iopt-1)
           else
             tb(itn) = tg_init
           endif
         endif

         temp = tb(itn)

         if (ta .gt. temp) then
           ta = temp
         endif
         if (tc .lt. temp) then
           tc = temp
         endif

         ta = dmax1(ta, tap)
         tc = dmin1(tc, tcp)

         call LADENS(ta, iopt)
         call FROID (ta, iopt)
         call CHAUD (ta, iopt, itn)
         btha = chotot(iopt) - reftot(iopt)

         call LADENS(tc, iopt)
         call FROID (tc, iopt)
         call CHAUD (tc, iopt, itn)
         bthc = chotot(iopt) - reftot(iopt)

         call LADENS(temp, iopt)
         call FROID (temp, iopt)
         call CHAUD (temp, iopt, itn)
         bthb = chotot(iopt) - reftot(iopt)
         energi = chotot(iopt) + reftot(iopt)
         bthrel = bthb/energi

         if ((btha*bthc .gt. zero)
     1        .and. (dabs(tcp - tap) .lt. 0.01d0)) then
           if (bthb .gt. zero) then
             tcp = tcp * 1.1d0
           else
             tap = tap * 0.9d0
           endif
         endif

       endif

       write(iscrn,*) 
       write(iscrn,*) '--------------- bilan thermique --------------'
       write (iscrn,1001) tap, ta, tb(itn), tc, tcp
     1                  , btha, bthrel, bthc

       endif

 1001  format (2x,5f10.2,2x,1p,3e12.3)

       return
       end

c-------------------------------------------------------------------

       subroutine LADENS(tabc, iopt)

       use constantes
       use auxiliaries
       use SP_DATA
       USE STR_DATA
       USE CHEM_DATA
       USE PROFIL

c   *** correction Pression, le 9 Aout 92: et 21 janv 94

       implicit double precision (a-h, o-z)

       save

       if (ifisob .ne. 2) then
         return
       endif

       hsurh2 = ab(i_h) / ab(i_h2)
       divi = (un + hsurh2) / (deux + hsurh2)
       xdens = (presse / tabc) / (abtot(5) + divi)

       do ii=1,nspec
         ab(ii) = ab(ii) * xdens / densh(iopt)
       end do
       densh(iopt) = xdens

       return
       end
